/**
 * Gestion de modes affichages
 *
 * Fonctionnement :
 *
 * Des boutons permettent de permuter des classes sur un élément cible.
 * Les modes d'affichages peuvent être persistants via un cookie `albums_affichages`.
 *
 * Utilisation :
 *
 * Il faut que les boutons soient présents dans le html, regroupés dans un conteneur (voir exemple plus bas).
 * À partir de là 2 possibilités pour activer la machine :
 *
 * 1) Tout gérer au moyen d'attributs data (**obligatoire)
 *    Sur le conteneur :
 *    - **data-affichages-cible (string) : sélecteur jQuery indiquant la cible → `#truc`, `.mon-bloc`, `[class*=machin]`, etc.
 *    - data-affichages-persistant (bool) : indique si l'affichage doit être persistant ou non via un cookie `albums_affichages` (défaut = true)
 *    - data-affichages-identifiant (string) : identifiant pour le cookie et les events (**obligatoire si persistant)
 *    Sur les boutons :
 *    - **data-classe (string) : la classe associée au bouton (peut-être vide)
 * 2) Lancer la fonction .albums_affichages manuellement sur le conteneur des boutons en indiquant la cible et cie :
 *    $('.affichages').albums_affichages(cible, options);
 *
 * Exemple :
 *
 * <div data-affichages-cible="#articles" data-affichages-identifiant="articles" class="affichages">
 *   <a data-affichages-classe="grand" class="sp-icone sp-icone_lorem" title="Vue large"></a>
 *   <a data-affichages-classe="petit" class="sp-icone sp-icone_ipsum" title="Vue rétrécie"></a>
 * </div>
 *
 * NB : les attributs d'accessibilité sont ajouté automatiquement aux boutons
 *
 * Évènements :
 *
 * 2 types d'évènements sont déclenchés :
 * - albums_affichages.change : chaque fois qu'on change le mode d'affichage
 * - albums_affichages.charge : quand on applique le mode d'affichage enregistré dans le cookie au chargement
 *
 * @param string cible
 *     sélecteur jQuery de la cible sur laquelle appliquer les classes
 * @param object options
 *     - persistant (bool)    : pour rendre le choix d'affichage persistant via un cookie `albums_affichages`
 *     - identifiant (string) : identifiant pour le cookie et les events (obligatoire si persistant)
 *                              on peut par exemple indiquer le type d'élément ciblé : documents, articles, etc.
 *     - classeActif (string) : classe à ajouter au bouton actif, défaut = 'on'
 */
$.albums_affichages = function(el, cible, options) {

	const me = this;

	// Initialisation
	// @return void
	this.init = function () {

		// La cible est donnée soit en param, soit dans les attributs data
		let $cible = $(cible);

		// Options : merger le défaut, celles passées en param et celles en attributs data
		let options_data = {};
		$.each($(el).data(), function (k, v) {
			// enlever le namespace 'affichages'
			let cle = k.replace('affichages', '');
			cle = cle.charAt(0).toLowerCase() + cle.slice(1);
			if (cle === 'cible') {
				$cible = $(v);
			} else {
				options_data[cle] = v;
			}
		});
		me.options = $.extend({}, $.albums_affichages.defaultOptions, options, options_data);

		// Garder les infos sous la main
		me.$controleur = $(el);                  // élément conteneur des boutons
		me.$cible      = $cible;                 // élément ciblé
		me.identifiant = me.options.identifiant; // identifiant pour le stockage et les events

		// préparer les boutons et les interactions (ajoute me.$boutons + me.classes)
		me.preparer_boutons();

		// À ce stade vérifier qu'on a tout ce qu'il faut
		const message_erreur = me.verifier_init();
		if (message_erreur) {
			console.log(message_erreur);
			return;
		}

		// Appliquer l'affichage enregistré dans le cookie
		me.charger_affichage();
	}

	// Vérifier qu'il y a tout ce qu'il faut
	this.verifier_init = function() {
		let erreur = '';
		if (me.$cible.length === 0) {
			erreur = 'cible absente';
		} else if (me.options.persistant === true && !me.identifiant) {
			erreur = 'identifiant obligatoire';
		} else if (!me.$boutons) {
			erreur = 'pas de bouton';
		}
		if (erreur) {
			erreur = '[albums_affichages] ' + erreur;
		}

		return erreur;
	}

	// Préparer les boutons : préparer le html, parser les classes, ajouter les écouteurs d'évènements
	// @return void
	this.preparer_boutons = function() {

		let $boutons = me.$controleur.find('[data-affichages-classe]');
		const classes = [];

		// Ajouter les attributs d'accessibilité
		me.$controleur.show().removeAttr('hidden');
		$.each($boutons, function (index) {
			const actif = $(this).hasClass(me.options.classeBtnActif) ? true : false;
			$(this)
				.attr('tabindex', actif ? 1 : 0)
				.attr('aria-pressed', actif ? true : false);
			if ($(this).prop('tagName') === 'A') {
				$(this).attr('role', 'button');
			}
			classes.push($(this).data('affichages-classe'));
		});

		me.$boutons = $boutons;
		me.classes = classes;

		// Actionner
		$boutons.click(function () {
			me.changer_affichage(this);
		});

	}

	// Change le mode d'affichage et le stocke dans un cookie
	// @param object el Bouton cliqué
	// @return void
	this.changer_affichage = function (el) {
		const
			cookie       = me.options.cookie,
			classe       = $(el).data('affichages-classe'),
			classes      = me.classes,
			classe_actif = me.options.classeBtnActif;

		// Classe et attributs sur les boutons
		$(el)
			.addClass(classe_actif).attr('aria-pressed', true).attr('tabindex', 1)
			.siblings().removeClass(classe_actif).attr('aria-pressed', false).attr('tabindex', 0);

		// Classe sur la cible
		// En jquery 3.3 on pourra passer un array à removeClass
		$.each(classes, function (i, c) {
			me.$cible.removeClass(c);
		});
		me.$cible.addClass(classe);

		// Poser le cookie
		if (me.options.persistant === true && me.identifiant) {
			var affichages = JSON.parse(Cookies.get(cookie) || '{}');
			affichages[me.identifiant] = classe;
			affichages = JSON.stringify(affichages);
			Cookies.set(cookie, affichages);
		}

		// Déclencher un évènement
		me.$cible.trigger('albums_affichages.change', {
			'cible':       me.$cible,
			'identifiant': me.identifiant,
			'element':     el,
			'classe':      classe,
		});

	}

	// Applique le mode d'affichage enregistré dans le cookie
	// @return void
	this.charger_affichage = function () {
		const
			cookie     = me.options.cookie,
			affichages = JSON.parse(Cookies.get(cookie) || '{}');

		if (me.identifiant in affichages) {
			var classe_cookie = affichages[me.identifiant];
			var $bouton = me.$boutons.filter(function () { return $(this).data('affichages-classe') === classe_cookie });
			$bouton.trigger('click');
		}
		me.$cible.trigger('albums_affichages.charge', {
			'cible':       me.$cible,
			'identifiant': me.identifiant,
			'classe':      classe_cookie,
		});
	}

	// Go go go
	this.init();
};

// Options par défaut
$.albums_affichages.defaultOptions = {
	identifiant: '',
	persistant: true,
	classeBtnActif: 'on',
	cookie: 'albums_affichages'
};

// Encapsuleur
(function ( $ ) {
	$.fn.albums_affichages = function(cible, options) {
		return this.each(function() {
			(new $.albums_affichages(this, cible, options));
		});
	};
}( jQuery ));

/* Lancer la fonction sur tous les éléments disposant des bons attributs */
function init_albums_affichages() {
	$('[data-affichages-cible]').albums_affichages();
}

/* Initialisation et relance en cas de chargement ajax */
if (window.jQuery) {
	jQuery(function($){
		init_albums_affichages();
		onAjaxLoad(init_albums_affichages);
	});
}